@@ -2,7 +2,7 @@
 #ifndef __GLADE_SIGNAL_H__
 #define __GLADE_SIGNAL_H__
 
-#include "glade-parser.h"
+#include <glib.h>
 
 G_BEGIN_DECLS
 
@@ -17,9 +17,6 @@ struct _GladeSignal
 	gchar    *name;         /* Signal name eg "clicked"            */
 	gchar    *handler;      /* Handler function eg "gtk_main_quit" */
 	gchar    *userdata;     /* User data signal handler argument   */
-	gboolean  lookup;       /* Whether user_data should be looked up
-				 * with the GModule interface by libglade.
-				 */
 	gboolean  after;        /* Connect after TRUE or FALSE         */
 };
 
@@ -27,18 +24,16 @@ struct _GladeSignal
 GladeSignal *glade_signal_new   (const gchar *name,
 				 const gchar *handler,
 				 const gchar *userdata,
-				 gboolean     lookup,
 				 gboolean     after);
 GladeSignal *glade_signal_clone (const GladeSignal *signal);
 void         glade_signal_free  (GladeSignal *signal);
 
 gboolean     glade_signal_equal (GladeSignal *sig1, GladeSignal *sig2);
 
-gboolean     glade_signal_write (GladeSignalInfo *info, GladeSignal *signal,
-				 GladeInterface *interface);
-
-GladeSignal *glade_signal_new_from_signal_info (GladeSignalInfo *info);
-
+GladeSignal *glade_signal_read  (GladeXmlNode *node);
+void         glade_signal_write (GladeSignal     *signal,
+				 GladeXmlContext *context,
+				 GladeXmlNode    *node);
 
 G_END_DECLS
 
